#!/usr/bin/perl   
#!/usr/local/bin/perl 
use Getopt::Std;
use lib ".";
use config;

#########################################################################
# grade-archlabb.pl - Architecture Lab Part B autograder
#
# Copyright (c) 2002, R. Bryant and D. O'Hallaron, All rights reserved.
# May not be used, modified, or copied without permission.
#########################################################################

# autoflush output on every print statement
$| = 1; 

# Any tmp files created by this script are readable only by the user
umask(0077); 

#
# usage - print help message and terminate
#
sub usage {
    printf STDERR "$_[0]\n";
    printf STDERR "Usage: $0 -f <file> [-he] [-s <codedir>]\n";
    printf STDERR "Options:\n";
    printf STDERR "  -h           Print this message.\n";
    printf STDERR "  -e           Don't include original handin file on the grade sheet\n";
    printf STDERR "  -f <file>    HCL file to be graded\n";
    printf STDERR "  -s <codedir> Directory where Y86 simulators are located\n";
    die "\n";
}

##############
# Main routine
##############

# 
# Parse the command line arguments
#
getopts('hef:s:');
if ($opt_h) {
    usage();
}
if (!$opt_f) {
    usage("Missing required argument (-f)");
}

# 
# These optional flags override defaults in config.pm
#
if ($opt_s) {         # simulator code directory
    $CODEDIR = $opt_s;
}

# 
# Initialize some file and path names
#
$infile = $opt_f;
($infile_basename = $infile) =~ s#.*/##s;     # basename of input files
$0 =~ s#.*/##s;                               # this prog's basename

# location of the temporary student distribution
$tmpdir = "/tmp/sim.$$";      

# absolute pathname of code directory
$codedir_abs = `cd $CODEDIR; pwd`; 
chomp($codedir_abs);

# 
# This is a message we use in several places when the program dies
#
$diemsg = "The files are in $tmpdir.";

# 
# Make sure the Y86 tools directory and the sssim binary exists
#
(-e $codedir_abs and -x $codedir_abs)
    or  die "$0: ERROR: Can't access $codedir_abs.\n";
(-e "$codedir_abs/seq/ssim" and -x "$codedir_abs/seq/ssim")
    or  die "$0: ERROR: Can't access $codedir_abs/seq/ssim\n";

# 
# Make sure the input file exists and is readable
#
open(INFILE, $infile) 
    or die "$0: ERROR: could not open file $infile\n";
close(INFILE);

# 
# Set up the contents of a scratch directory with a student
# distribution of the Y86 tools
#
system("mkdir $tmpdir") == 0
  or die "ERROR: mkdir $tmpdir failed\n";
system("(cd $codedir_abs; make -s DISTDIR=$tmpdir/sim dist)") == 0
  or die "ERROR: Could not create Y86 tools in $tmpdir/sim\n";
system("(cd $tmpdir/sim; make -s clean; make -s)") == 0
  or die "ERROR: Could not make baseline simulator\n";


# Print header
print "\nCS:APP Architecture Lab Part B: Grading Sheet for $infile_basename\n\n";

#
# Compile the HCL file and link into the simulator
# 
print "Part 1: Compiling simulator using $infile\n\n";
system("(cp $infile $tmpdir/sim/seq/seq-full.hcl)") == 0
  or die "ERROR: Could not copy input file $infile\n";
system("(cd $tmpdir/sim/seq; make VERSION=full ssim)") == 0
  or die "ERROR: Could not make simulator\n";

#
# Run benchmark regression tests
#
print "\n\nPart 2: Benchmark regression tests\n\n";
system("(cd $tmpdir/sim/y86-code; make testssim)") == 0
  or die "ERROR: Could not run benchmark regression tests\n";

#
# Run synthetic regression tests for iaddl
#
print "\n\nPart 3: Synthetic regression tests for iaddl\n\n";
system("(cd $tmpdir/sim/ptest; make SIM=../seq/ssim TFLAGS=-i)") == 0
  or die "ERROR: Could not run synthetic regression tests for iaddl\n";

#
# Run synthetic regression tests for leave
#
print "\n\nPart 4: Synthetic regression tests for leave\n\n";
system("(cd $tmpdir/sim/ptest; make SIM=../seq/ssim TFLAGS=-l)") == 0
  or die "ERROR: Could not run synthetic regression tests leave\n";

# 
# Optionally print the original handin file 
#
if (!$opt_e) {
    print "\f\nPart 5: Handin file $infile_basename\n\n";
    system("cat $infile");
} 

#
# Everything went OK, so remove the scratch directory
#
system("rm -fr $tmpdir");

exit;


