#!/usr/bin/perl   
#!/usr/local/bin/perl 
use Getopt::Std;
use lib ".";
use config;

#########################################################################
# grade-archlabc.pl - Architecture Lab Part C autograder
# 
# Prints a grade sheet for one student's handin.
#
# Copyright (c) 2002, R. Bryant and D. O'Hallaron, All rights reserved.
# May not be used, modified, or copied without permission.
#########################################################################

# autoflush output on every print statement
$| = 1; 

# Any tmp files created by this script are readable only by the user
umask(0077); 

#
# usage - print help message and terminate
#
sub usage {
    printf STDERR "$_[0]\n";
    printf STDERR "Usage: $0 -f <file> [-he] [-s <codedir>]\n";
    printf STDERR "Options:\n";
    printf STDERR "  -h           Print this message.\n";
    printf STDERR "  -e           Don't include original handin file on the grade sheet\n";
    printf STDERR "  -f <file>    Prefix of handin files to be graded (e.g., bovik-1)\n";
    printf STDERR "  -s <codedir> Directory where Y86 simulators are located\n";
    die "\n";
}

##############
# Main routine
##############

# 
# Parse the command line arguments
#
getopts('hef:s:');
if ($opt_h) {
    usage();
}
if (!$opt_f) {
    usage("Missing required argument (-f)");
}

# 
# These optional flags override defaults in config.pm
#
if ($opt_s) {  # simulator code directory
    $CODEDIR = $opt_s;
}

# 
# Initialize some file and path names
#
$infile = $opt_f;   # This is really the root name of the set of handins
($infile_basename = $infile) =~ s#.*/##s;     # basename of input files
$tmpdir = "/tmp/$infile_basename.$$";         # scratch directory
$0 =~ s#.*/##s;                               # this prog's basename

# absolute pathname of code directory
$codedir_abs = `cd $CODEDIR; pwd`; 
chomp($codedir_abs);

# 
# This is a message we use in several places when the program dies
#
$diemsg = "The files are in $tmpdir.";

# 
# Make sure the code directory and the psim binary exist
#
(-e $codedir_abs and -x $codedir_abs)
    or  die "$0: ERROR: Can't access $codedir_abs.\n";
(-e "$codedir_abs/pipe/psim" and -x "$codedir_abs/pipe/psim")
    or  die "$0: ERROR: Can't access $codedir_abs/pipe/psim.\n";

# 
# Make sure the input files exist and are readable
#
open(INFILE, "$infile-pipe-full.hcl") 
    or die "$0: ERROR: could not open file $infile-pipe-full.hcl\n";
close(INFILE);
open(INFILE, "$infile-ncopy.ys") 
    or die "$0: ERROR: could not open file $infile-ncopy.ys\n";
close(INFILE);

# 
# Set up the contents of a scratch directory with a student
# distribution of the Y86 tools
#
system("mkdir $tmpdir") == 0
  or die "ERROR: mkdir $tmpdir failed\n";
system("(cd $codedir_abs; make -s DISTDIR=$tmpdir/sim dist)") == 0
  or die "ERROR: Could not create Y86 tools in $tmpdir/sim\n";
system("(cd $tmpdir/sim; make -s clean; make -s)") == 0
  or die "ERROR: Could not make baseline simulator\n";


# Print grade sheet header
print "\nCS:APP Architecture Lab Part C: Grading Sheet for $infile_basename\n\n";

#
# Build the student's simulator and copy the student's ncopy.ys file
# 
print "Part 1: Building simulator\n\n";
system("(cp $infile-pipe-full.hcl $tmpdir/sim/pipe/pipe-full.hcl)") == 0
  or die "ERROR: Could not copy HCL file\n";
system("(cd $tmpdir/sim/pipe; make VERSION=full psim)") == 0
  or die "ERROR: Could not make simulator\n";

system("(cp $infile-ncopy.ys $tmpdir/sim/pipe/ncopy.ys)") == 0
  or die "ERROR: Could not input file\n";

#
# Run benchmark regression tests on simulator
#
print "\n\nPart 2: Benchmark regression tests\n\n";
system("(cd $tmpdir/sim/y86-code; make testpsim)") == 0
  or die "ERROR: Could not run benchmark regression tests\n";

#
# Run synthetic regression tests on simulator
#
print "\n\nPart 3: Synthetic regression tests\n\n";
system("(cd $tmpdir/sim/ptest; make SIM=../pipe/psim TFLAGS=)") == 0
  or die "ERROR: Could not run synthetic regression tests\n";

#
# Run correctness tests on the student's ncopy.ys program
#
print "\n\nPart 4: Correctness of ncopy program\n\n";
system("(cd $tmpdir/sim/pipe; ./correctness.pl -f ncopy.ys)") == 0
  or die "ERROR: Could not run ncopy correctness tests\n";

#
# Run performance tests on the student's ncopy.ys program
#
print "\n\nPart 5: Performance test\n\n";
system("(cd $tmpdir/sim/pipe; ./benchmark.pl -f ncopy.ys)") == 0
  or die "ERROR: Could not run ncopy performance tests\n";

# 
# Optionally print the original handin files 
#
if (!$opt_e) {
    print "\f\nPart 6: Handin file $infile_basename-ncopy.ys\n\n";
    system("cat $infile-ncopy.ys");

    print "\f\nPart 7: Handin file $infile_basename-pipe-full.hcl\n\n";
    system("cat $infile-pipe-full.hcl");
} 

#
# Everything went OK, so remove the scratch directory
#
system("rm -fr $tmpdir");

exit;


