$c	This is the master version of the .hcl code for the pipeline.
$c	The Perl script ../hcl/vextract.pl extracts the different versions
$c	depending on the options shown in the first characters of each line.
$c	vextract.pl is given a set of single-character version names.
$c	Any line beginning with $[a-zA-Z]*\t is included only if
$c	one of the characters matches one of the version names.
$c	Any line beginning with $![a-zA-Z]*\t is included unless
$c	one of the characters matches one of the version names.
$c	
$c	Options:
$c	c: Comments (won't show up in any extracted version)
$c	a: Add annotations (used for generating .tex files)
$c	b: Broken pipeline.  Does not handle any hazards
$c	f: iaddl+leave problem
$c	F: iaddl+leave solution
$c	l: Load forward problem
$c	L: Load forward solution
$c	n: Branches not-taken problem
$c	N: Branches not-taken solution
$c	t: Backward taken, forward-not taken problem
$c	T: Backward taken, forward-not taken solution
$c	w: Single write port problem
$c	W: Single write port solution
$c	s: Stall-based pipeline control problem
$c	S: Stall-based pipeline control solution
#/* $begin pipe-all-hcl */
####################################################################
#    HCL Description of Control for Pipelined Y86 Processor        #
#    Copyright (C) Randal E. Bryant, David R. O'Hallaron, 2002     #
####################################################################
$fF	
$f	## Your task is to implement the iaddl and leave instructions
$f	## The file contains a declaration of the icodes
$f	## for iaddl (IIADDL) and leave (ILEAVE).
$f	## Your job is to add the rest of the logic to make it work
$F	## This is the solution for the iaddl and leave problems
$lL	
$l	## Your task is to implement load-forwarding, where a value
$l	## read from memory can be stored to memory by the immediately
$l	## following instruction without stalling
$l	## This requires introducing an explicit declaration of new_M_valA
$l	## and relaxing the stall conditions.  Relevant sections to change
$l	## are shown in comments containing the keyword "LB"
$L	## This is the solution to the load-forwarding problem
$nN	
$n	## Your task is to modify the design so that conditional branches are
$n	## predicted as being not-taken.  The code here is nearly identical
$n	## to that for the normal pipeline.  
$n	## Comments starting with keyword "BNT" have been added at places
$n	## relevant to the exercise.
$N	## This is the solution for the branches not-taken problem
$tT	
$t	## Your task is to modify the design so that conditional branches are
$t	## predicted as being taken when backward and not-taken when forward
$t	## The code here is nearly identical to that for the normal pipeline.  
$t	## Comments starting with keyword "BBTFNT" have been added at places
$t	## relevant to the exercise.
$T	## BBTFNT: This is the solution for the backward taken, forward
$T	## not-taken branch prediction problem
$wW	
$w	## Your task is to modify the design so that on any cycle, only
$w	## one of the two possible (valE and valM) register writes will occur.
$w	## This requires special handling of the popl instruction.
$W	## This is a solution to the single write port problem
$wW	## Overall strategy:  IPOPL passes through pipe, 
$wW	## treated as stack pointer increment, but not incrementing the PC
$wW	## On refetch, modify fetched icode to indicate an instruction "IPOP2",
$wW	## which reads from memory.
$w	## This requires explicit declaration of the value new_D_icode
$w	## and lots of other changes.  Relevant positions to change
$w	## are indicated by comments starting with keyword "1W".
$sS	
$s	## Your task is to make the pipeline work without using any forwarding
$s	## The normal bypassing logic in the file is disabled.
$s	## You can only change the pipeline control logic at the end of this file.
$s	## The trick is to make the pipeline stall whenever there is a data hazard.
$S	## This is the solution for the stall-based pipeline control problem.
$b	## This version does not detect or handle any hazards

####################################################################
#    C Include's.  Don't alter these                               #
####################################################################

quote '#include <stdio.h>'
quote '#include "isa.h"'
quote '#include "pipeline.h"'
quote '#include "stages.h"'
quote '#include "sim.h"'
quote 'int sim_main(int argc, char *argv[]);'
quote 'int main(int argc, char *argv[]){return sim_main(argc,argv);}'

####################################################################
#    Declarations.  Do not change/remove/delete any of these       #
####################################################################

##### Symbolic representation of Y86 Instruction Codes #############
intsig INOP 	'I_NOP'
intsig IHALT	'I_HALT'
intsig IRRMOVL	'I_RRMOVL'
intsig IIRMOVL	'I_IRMOVL'
intsig IRMMOVL	'I_RMMOVL'
intsig IMRMOVL	'I_MRMOVL'
intsig IOPL	'I_ALU'
intsig IJXX	'I_JMP'
intsig ICALL	'I_CALL'
intsig IRET	'I_RET'
intsig IPUSHL	'I_PUSHL'
intsig IPOPL	'I_POPL'
$fF	# Instruction code for iaddl instruction
$fF	intsig IIADDL	'I_IADDL'
$fF	# Instruction code for leave instruction
$fF	intsig ILEAVE	'I_LEAVE'
$wW	# 1W: Special instruction code for second try of popl
$wW	intsig IPOP2	'I_POP2'

##### Symbolic representation of Y86 Registers referenced explicitly #####
intsig RESP     'REG_ESP'    	# Stack Pointer
$fF	intsig REBP     'REG_EBP'    	# Frame Pointer
intsig RNONE    'REG_NONE'   	# Special value indicating "no register"

##### ALU Functions referenced explicitly ##########################
intsig ALUADD	'A_ADD'		# ALU should add its arguments
$nNtT
$nN	## BNT: For modified branch prediction, need to distinguish
$tT	## BBTFNT: For modified branch prediction, need to distinguish
$nNtT	## conditional vs. unconditional branches
$nNtT	##### Jump conditions referenced explicitly
$nNtT	intsig JUNCOND 'J_YES'          # Code for unconditional jump instruction

##### Signals that can be referenced by control logic ##############

##### Pipeline Register F ##########################################

intsig F_predPC 'pc_curr->pc'		# Predicted value of PC

##### Intermediate Values in Fetch Stage ###########################

intsig f_icode	'if_id_next->icode'  # Fetched instruction code
intsig f_ifun	'if_id_next->ifun'   # Fetched instruction function
intsig f_valC	'if_id_next->valc'   # Constant data of fetched instruction
intsig f_valP	'if_id_next->valp'   # Address of following instruction
$wW	## 1W: Provide access to the PC value for the current instruction
$wW	intsig f_pc	'f_pc'               # Address of fetched instruction

##### Pipeline Register D ##########################################
intsig D_icode 'if_id_curr->icode'	# Instruction code
intsig D_rA 'if_id_curr->ra'	# rA field from instruction
intsig D_rB 'if_id_curr->rb'	# rB field from instruction
intsig D_valP 'if_id_curr->valp'	# Incremented PC

##### Intermediate Values in Decode Stage  #########################

intsig d_srcA	 'id_ex_next->srca'	# srcA from decoded instruction
intsig d_srcB	 'id_ex_next->srcb'	# srcB from decoded instruction
intsig d_rvalA 'd_regvala'		# valA read from register file
intsig d_rvalB 'd_regvalb'		# valB read from register file

##### Pipeline Register E ##########################################
intsig E_icode 'id_ex_curr->icode'	# Instruction code
intsig E_ifun  'id_ex_curr->ifun'       # Instruction function
intsig E_valC  'id_ex_curr->valc'	# Constant data
intsig E_srcA  'id_ex_curr->srca'       # Source A register ID
intsig E_valA  'id_ex_curr->vala'       # Source A value
intsig E_srcB  'id_ex_curr->srcb'       # Source B register ID
intsig E_valB  'id_ex_curr->valb'       # Source B value
intsig E_dstE 'id_ex_curr->deste'	# Destination E register ID
intsig E_dstM 'id_ex_curr->destm'	# Destination M register ID

##### Intermediate Values in Execute Stage #########################
intsig e_valE 'ex_mem_next->vale'	# valE generated by ALU
boolsig e_Bch 'ex_mem_next->takebranch' # Am I about to branch?

##### Pipeline Register M                  #####
intsig M_icode 'ex_mem_curr->icode'	# Instruction code
intsig M_ifun  'ex_mem_curr->ifun'	# Instruction function
intsig M_valA  'ex_mem_curr->vala'      # Source A value
intsig M_dstE 'ex_mem_curr->deste'	# Destination E register ID
intsig M_valE  'ex_mem_curr->vale'      # ALU E value
intsig M_dstM 'ex_mem_curr->destm'	# Destination M register ID
boolsig M_Bch 'ex_mem_curr->takebranch'	# Branch Taken flag
$lL	## LF: Carry srcA up to pipeline register M
$lL	intsig M_srcA 'ex_mem_curr->srca'	# Source A register ID

##### Intermediate Values in Memory Stage ##########################
intsig m_valM 'mem_wb_next->valm'	# valM generated by memory

##### Pipeline Register W ##########################################
intsig W_icode 'mem_wb_curr->icode'	# Instruction code
intsig W_dstE 'mem_wb_curr->deste'	# Destination E register ID
intsig W_valE  'mem_wb_curr->vale'      # ALU E value
intsig W_dstM 'mem_wb_curr->destm'	# Destination M register ID
intsig W_valM  'mem_wb_curr->valm'	# Memory M value

####################################################################
#    Control Signal Definitions.                                   #
####################################################################

################ Fetch Stage     ###################################

## What address should instruction be fetched at
$a	#/* $begin pipe-f_pc-hcl */
int f_pc = [
	# Mispredicted branch.  Fetch at incremented PC
$!NT		M_icode == IJXX && !M_Bch : M_valA;
$N		# BNT: Changed misprediction condition
$N		M_icode == IJXX && M_ifun != JUNCOND && M_Bch : M_valE;
$T		# BBTFNT: Mispredicted forward branch.  Fetch at target (now in valE)
$T	        M_icode == IJXX && M_ifun != JUNCOND && M_valE >= M_valA
$T	   	   && M_Bch : M_valE;
$T		# BBTFNT: Mispredicted backward branch.
$T		#    Fetch at incremented PC (now in valE)
$T	 	M_icode == IJXX && M_ifun != JUNCOND && M_valE < M_valA
$T		  && !M_Bch : M_valA;
	# Completion of RET instruction.
	W_icode == IRET : W_valM;
	# Default: Use predicted value of PC
	1 : F_predPC;
];
$a	#/* $end pipe-f_pc-hcl */

# Does fetched instruction require a regid byte?
bool need_regids =
	f_icode in { IRRMOVL, IOPL, IPUSHL, IPOPL, 
$W			     IPOP2,
$!F			     IIRMOVL, IRMMOVL, IMRMOVL };
$F			     IIRMOVL, IRMMOVL, IMRMOVL, IIADDL };

# Does fetched instruction require a constant word?
bool need_valC =
$!F		f_icode in { IIRMOVL, IRMMOVL, IMRMOVL, IJXX, ICALL };
$F		f_icode in { IIRMOVL, IRMMOVL, IMRMOVL, IJXX, ICALL, IIADDL };

bool instr_valid = f_icode in 
	{ INOP, IHALT, IRRMOVL, IIRMOVL, IRMMOVL, IMRMOVL,
$!FW		       IOPL, IJXX, ICALL, IRET, IPUSHL, IPOPL };
$F		       IOPL, IJXX, ICALL, IRET, IPUSHL, IPOPL, IIADDL, ILEAVE };
$W		       IOPL, IJXX, ICALL, IRET, IPUSHL, IPOPL, IPOP2 };

# Predict next value of PC
$a	#/* $begin pipe-new_F_predPC-hcl */
int new_F_predPC = [
$n		# BNT: This is where you'll change the branch prediction rule
$t		# BBTFNT: This is where you'll change the branch prediction rule
$!TN		f_icode in { IJXX, ICALL } : f_valC;
$N		# BNT: Revised branch prediction rule:
$N		#   Unconditional branch is taken, others not taken
$N		f_icode == IJXX && f_ifun == JUNCOND : f_valC;
$NT		f_icode in { ICALL } : f_valC;
$T		f_icode == IJXX && f_ifun == JUNCOND : f_valC; # Unconditional branch
$T		f_icode == IJXX && f_valC < f_valP : f_valC; # Backward branch
$w		## 1W: Want to refetch popl one time
$W		# First time through popl. Refetch popl
$W		f_icode == IPOPL && D_icode != IPOPL: f_pc;  
$T		# BBTFNT: Forward conditional branches will default to valP
	1 : f_valP;
];
$a	#/* $end pipe-new_F_predPC-hcl */

$wW	## W1: To split ipopl into two cycles, need to be able to 
$wW	## modify fetched value of icode, so that it will be IPOP2
$wW	## when fetched for second time.
$wW	# Set code for fetched instruction
$wW	int new_D_icode = [
$W		## Can detected refetch of ipopl, since now have
$W		## IPOPL as icode for instruction in decode.
$W		f_icode == IPOPL && D_icode == IPOPL : IPOP2;
$wW		1 : f_icode;
$wW	];

################ Decode Stage ######################################

$wW	## W1: Strategy.  Decoding of popl rA should be treated the same
$wW	## as would iaddl $4, %esp
$wW	## Decoding of pop2 rA treated same as mrmovl -4(%esp), rA

## What register should be used as the A source?
$a	#/* $begin pipe-new_E_srcA-hcl */
int new_E_srcA = [
	D_icode in { IRRMOVL, IRMMOVL, IOPL, IPUSHL  } : D_rA;
	D_icode in { IPOPL, IRET } : RESP;
$F		D_icode in { ILEAVE } : REBP;
	1 : RNONE; # Don't need register
];
$a	#/* $end pipe-new_E_srcA-hcl */

## What register should be used as the B source?
int new_E_srcB = [
$!F		D_icode in { IOPL, IRMMOVL, IMRMOVL  } : D_rB;
$F		D_icode in { IOPL, IRMMOVL, IMRMOVL, IIADDL  } : D_rB;
$!W		D_icode in { IPUSHL, IPOPL, ICALL, IRET } : RESP;
$W		D_icode in { IPUSHL, IPOPL, ICALL, IRET, IPOP2 } : RESP;
$F		D_icode in { ILEAVE } : REBP;
	1 : RNONE;  # Don't need register
];

## What register should be used as the E destination?
$a	#/* $begin pipe-new_E_dstE-hcl */
int new_E_dstE = [
$!F		D_icode in { IRRMOVL, IIRMOVL, IOPL} : D_rB;
$F		D_icode in { IRRMOVL, IIRMOVL, IOPL, IIADDL } : D_rB;
$!F		D_icode in { IPUSHL, IPOPL, ICALL, IRET } : RESP;
$F		D_icode in { IPUSHL, IPOPL, ICALL, IRET, ILEAVE } : RESP;
	1 : RNONE;  # Don't need register
];
$a	#/* $end pipe-new_E_dstE-hcl */

## What register should be used as the M destination?
int new_E_dstM = [
$!W	D_icode in { IMRMOVL, IPOPL } : D_rA;
$W	D_icode in { IMRMOVL, IPOP2 } : D_rA;
$F		D_icode in { ILEAVE } : REBP;
	1 : RNONE;  # Don't need register
];

## What should be the A value?
$!bsS	## Forward into decode stage for valA
$a	#/* $begin pipe-nobypass-hcl */
$a	#/* $begin pipe-new_E_valA-hcl */
$sS	##  DO NOT MODIFY THE FOLLOWING CODE.
$bsS	## No forwarding.  valA is either valP or value from register file
int new_E_valA = [
	D_icode in { ICALL, IJXX } : D_valP; # Use incremented PC
$!bsS		d_srcA == E_dstE : e_valE;    # Forward valE from execute
$!bsS		d_srcA == M_dstM : m_valM;    # Forward valM from memory
$!bsS		d_srcA == M_dstE : M_valE;    # Forward valE from memory
$!bsS		d_srcA == W_dstM : W_valM;    # Forward valM from write back
$!bsS		d_srcA == W_dstE : W_valE;    # Forward valE from write back
	1 : d_rvalA;  # Use value read from register file
];
$a	#/* $end pipe-new_E_valA-hcl */

$!bsS## Forward into decode stage for valB
$a	#/* $begin pipe-new_E_valB-hcl */
$!bsS	int new_E_valB = [
$!bsS		d_srcB == E_dstE : e_valE;    # Forward valE from execute
$!bsS		d_srcB == M_dstM : m_valM;    # Forward valM from memory
$!bsS		d_srcB == M_dstE : M_valE;    # Forward valE from memory
$!bsS		d_srcB == W_dstM : W_valM;    # Forward valM from write back
$!bsS		d_srcB == W_dstE : W_valE;    # Forward valE from write back
$!bsS		1 : d_rvalB;  # Use value read from register file
$!bsS	];
$bsS	## No forwarding.  valB is value from register file
$bsS	int new_E_valB = d_rvalB;
$a	#/* $end pipe-new_E_valB-hcl */
$a	#/* $end pipe-nobypass-hcl */

################ Execute Stage #####################################
$nNtT	
$nN	# BNT: When some branches are predicted as not-taken, you need some
$tT	# BBTFNT: When some branches are predicted as not-taken, you need some
$ntNT	# way to get valC into pipeline register M, so that
$ntNT	# you can correct for a mispredicted branch.
$NT	# One way to do this is to run valC through the ALU, adding 0
$NT	# so that valC will end up in M_valE

## Select input A to ALU
int aluA = [
	E_icode in { IRRMOVL, IOPL } : E_valA;
$!FNT		E_icode in { IIRMOVL, IRMMOVL, IMRMOVL } : E_valC;
$F		E_icode in { IIRMOVL, IRMMOVL, IMRMOVL, IIADDL } : E_valC;
$N		# BNT: Use ALU to pass E_valC to M_valE
$T		# BBTFNT: Use ALU to pass E_valC to M_valE
$NT		E_icode in { IIRMOVL, IRMMOVL, IMRMOVL, IJXX } : E_valC;
$!W		E_icode in { ICALL, IPUSHL } : -4;
$W		E_icode in { ICALL, IPUSHL, IPOP2 } : -4;
$!F		E_icode in { IRET, IPOPL } : 4;
$F		E_icode in { IRET, IPOPL, ILEAVE } : 4;
	# Other instructions don't need ALU
];

## Select input B to ALU
int aluB = [
	E_icode in { IRMMOVL, IMRMOVL, IOPL, ICALL, 
$!FW			      IPUSHL, IRET, IPOPL } : E_valB;
$F			      IPUSHL, IRET, IPOPL, IIADDL, ILEAVE } : E_valB;
$W			      IPUSHL, IRET, IPOPL, IPOP2 } : E_valB;
$N		# BNT: Add 0 to valC
$T		# BBTFNT: Add 0 to valC
$!NT		E_icode in { IRRMOVL, IIRMOVL } : 0;
$NT		E_icode in { IRRMOVL, IIRMOVL, IJXX } : 0;
	# Other instructions don't need ALU
];

## Set the ALU function
int alufun = [
	E_icode == IOPL : E_ifun;
	1 : ALUADD;
];

## Should the condition codes be updated?
$!F	bool set_cc = E_icode == IOPL;
$F	bool set_cc = E_icode in { IOPL, IIADDL };

$lL	
$lL	## Generate M_valA
$lL	## LB: With load forwarding, want to insert valM 
$lL	##   from memory stage when appropriate
$l	## Here it is set to the default used in the normal pipeline
$lL	int new_M_valA = [
$L		# Forwarding Condition
$L		M_dstM == E_srcA && E_icode in { IPUSHL, IRMMOVL } : m_valM;
$lL		# Use valA
$lL		1 : E_valA;
$lL	];

################ Memory Stage ######################################

## Select memory address
int mem_addr = [
$!W	M_icode in { IRMMOVL, IPUSHL, ICALL, IMRMOVL } : M_valE;
$W	M_icode in { IRMMOVL, IPUSHL, ICALL, IMRMOVL, IPOP2 } : M_valE;
$!FW		M_icode in { IPOPL, IRET } : M_valA;
$F		M_icode in { IPOPL, IRET, ILEAVE } : M_valA;
$W		M_icode in { IRET } : M_valA;
	# Other instructions don't need address
];

## Set read control signal
$!FW	bool mem_read = M_icode in { IMRMOVL, IPOPL, IRET };
$F	bool mem_read = M_icode in { IMRMOVL, IPOPL, IRET, ILEAVE };
$W	bool mem_read = M_icode in { IMRMOVL, IPOP2, IRET };

## Set write control signal
bool mem_write = M_icode in { IRMMOVL, IPUSHL, ICALL };

$wW	################ Write back stage ##################################
$wW	
$wW	## 1W: For this problem, we introduce a multiplexor that merges
$wW	## valE and valM into a single value for writing to register port E.
$wW	## DO NOT CHANGE THIS LOGIC
$wW
$wW	## Merge both write back sources onto register port E 
$a	/* $begin pipe-1w-wbe-hcl */
$wW	int w_dstE = [
$wW		## writing from valM
$wW		W_dstM != RNONE : W_dstM;
$wW		1: W_dstE;
$wW	];
$wW
$wW	int w_valE = [
$wW		W_dstM != RNONE : W_valM;
$wW		1: W_valE;
$wW	];
$a	/* $end pipe-1w-wbe-hcl */
$wW
$wW	## Set so that register port M is never used.
$a	/* $begin pipe-1w-wbm-hcl */
$wW	int w_dstM = RNONE;
$wW
$wW	int w_valM = 0;
$a	/* $end pipe-1w-wbm-hcl */

################ Pipeline Register Control #########################

$S	#/* $begin pipe-nobypass-cntl-hcl */
# Should I stall or inject a bubble into Pipeline Register F?
# At most one of these can be true.
bool F_bubble = 0;
$a	#/* $begin pipe-F-cntl-hcl */
bool F_stall =
$b		0;
$!bsS		# Conditions for a load/use hazard
$l		## Set this to the new load/use condition
$l		0 ||
$!FlLWbsS		E_icode in { IMRMOVL, IPOPL } &&
$F		E_icode in { IMRMOVL, IPOPL, ILEAVE } &&
$L		E_icode in { IMRMOVL, IPOPL } &&
$L		 (E_dstM == d_srcB ||
$L	          (E_dstM == d_srcA && !D_icode in { IRMMOVL, IPUSHL })) ||
$W		E_icode in { IMRMOVL, IPOP2 } &&
$!lLbsS		 E_dstM in { d_srcA, d_srcB } ||
$s		# Modify the following to stall the update of pipeline register F
$s		0 ||
$S		# Stall if either operand source is destination of 
$S		# instruction in execute, memory, or write-back stages
$S		d_srcA != RNONE && d_srcA in 
$S		  { E_dstM, E_dstE, M_dstM, M_dstE, W_dstM, W_dstE } ||
$S		d_srcB != RNONE && d_srcB in 
$S		  { E_dstM, E_dstE, M_dstM, M_dstE, W_dstM, W_dstE } ||
$!b		# Stalling at fetch while ret passes through pipeline
$!b		IRET in { D_icode, E_icode, M_icode };
$a	#/* $end pipe-F-cntl-hcl */

# Should I stall or inject a bubble into Pipeline Register D?
# At most one of these can be true.
$a	#/* $begin pipe-D-stall-hcl */
bool D_stall = 
$b		0;
$!bsS		# Conditions for a load/use hazard
$l		## Set this to the new load/use condition
$l		0; 
$!lFBWbsS		E_icode in { IMRMOVL, IPOPL } &&
$F		E_icode in { IMRMOVL, IPOPL, ILEAVE } &&
$L		E_icode in { IMRMOVL, IPOPL } &&
$L		 (E_dstM == d_srcB ||
$L	          (E_dstM == d_srcA && !D_icode in { IRMMOVL, IPUSHL }));
$W		E_icode in { IMRMOVL, IPOP2 } &&
$!bsSlL		 E_dstM in { d_srcA, d_srcB };
$s		# Modify the following to stall the instruction in decode
$s		0;
$S		# Stall if either operand source is destination of 
$S		# instruction in execute, memory, or write-back stages
$S		# but not part of mispredicted branch
$S		!(E_icode == IJXX && !e_Bch) &&
$S		 (d_srcA != RNONE && d_srcA in 
$S		    { E_dstM, E_dstE, M_dstM, M_dstE, W_dstM, W_dstE } ||
$S		  d_srcB != RNONE && d_srcB in 
$S		    { E_dstM, E_dstE, M_dstM, M_dstE, W_dstM, W_dstE });
$a	#/* $end pipe-D-stall-hcl */

$a	#/* $begin pipe-D-bubble-hcl */
bool D_bubble =
$b		0;
$!b		# Mispredicted branch
$N		# BNT: Changed misprediction condition
$!bNT		(E_icode == IJXX && !e_Bch) ||
$N		(E_icode == IJXX && E_ifun != JUNCOND && e_Bch) ||
$t		# BBTFNT: This condition will change
$T		# BBTFNT: Changed misprediction condition
$T		(E_icode == IJXX && E_ifun != JUNCOND &&
$T	          (E_valC < E_valA && !e_Bch || E_valC >= E_valA && e_Bch)) ||
$!b		# Stalling at fetch while ret passes through pipeline
$!bsS		# but not condition for a load/use hazard
$!bF		!(E_icode in { IMRMOVL, IPOPL } && E_dstM in { d_srcA, d_srcB }) &&
$F		!(E_icode in { IMRMOVL, IPOPL, ILEAVE } && E_dstM in { d_srcA, d_srcB }) &&
$sS		# but not condition for a generate/use hazard
$s		!0 &&
$S		!(d_srcA != RNONE && d_srcA in 
$S		   { E_dstM, E_dstE, M_dstM, M_dstE, W_dstM, W_dstE } ||
$S		  d_srcB != RNONE && d_srcB in 
$S		   { E_dstM, E_dstE, M_dstM, M_dstE, W_dstM, W_dstE }) &&
$!b		  IRET in { D_icode, E_icode, M_icode };
$a	#/* $end pipe-D-bubble-hcl */

# Should I stall or inject a bubble into Pipeline Register E?
# At most one of these can be true.
bool E_stall = 0;
$a	#/* $begin pipe-E-bubble-hcl */
bool E_bubble =
$b		0;
$!b		# Mispredicted branch
$N		# BNT: Changed misprediction condition
$!bNT		(E_icode == IJXX && !e_Bch) ||
$N		(E_icode == IJXX && E_ifun != JUNCOND && e_Bch) ||
$t		# BBTFNT: This condition will change
$T		# BBTFNT: Changed misprediction condition
$T		(E_icode == IJXX && E_ifun != JUNCOND &&
$T	          (E_valC < E_valA && !e_Bch || E_valC >= E_valA && e_Bch)) ||
$!bsS		# Conditions for a load/use hazard
$l		## Set this to the new load/use condition
$l		0;
$!FlLWbsS		E_icode in { IMRMOVL, IPOPL } &&
$F		E_icode in { IMRMOVL, IPOPL, ILEAVE } &&
$L		E_icode in { IMRMOVL, IPOPL } &&
$L		 (E_dstM == d_srcB ||
$L	          (E_dstM == d_srcA && !D_icode in { IRMMOVL, IPUSHL }));
$W		E_icode in { IMRMOVL, IPOP2 } &&
$!lLbsS		   E_dstM in { d_srcA, d_srcB};
$s		# Modify the following to inject bubble into the execute stage
$s		0;
$S		  # Inject bubble if either operand source is destination of 
$S		  # instruction in execute, memory, or write back stages
$S		  d_srcA != RNONE && 
$S			d_srcA in { E_dstM, E_dstE, M_dstM, M_dstE, W_dstM, W_dstE } ||
$S		  d_srcB != RNONE && 
$S			d_srcB in { E_dstM, E_dstE, M_dstM, M_dstE, W_dstM, W_dstE };
$a	#/* $end pipe-E-bubble-hcl */

# Should I stall or inject a bubble into Pipeline Register M?
# At most one of these can be true.
bool M_stall = 0;
bool M_bubble = 0;
$S	#/* $end pipe-nobypass-cntl-hcl */
$a	#/* $end pipe--hcl */
#/* $end pipe-all-hcl */
