// --------------------------------------------------------------------
// This part of the code was generated by including the file "pipe-proc.v"
// --------------------------------------------------------------------
// Verilog representation of PIPE
// This file connects together all of the blocks to create the processor

// The tag INSERT-LOGIC-HERE (but with underscores rather than dashes)
// is used to indicate where the logic derived from the HCL code should be inserted.

// Tags of the form OMIT_XX are used to strip away default assignments for homework problems
// 4.41 & 4.42

// The processor can run in 5 different modes:
// RUN: Normal operation
// RESET: sets PC to 0, clears all pipe registers, initializes condition codes
// DOWNLOAD: Download bytes from controller into memory
// UPLOAD: Upload bytes from memory to controller
// STATUS: Upload other status information to controller

// Processor module
module processor(mode, udaddr, idata, odata, status, clock);
  input  [2:0]  mode;   // Signal operating mode to processor
  input  [31:0] udaddr; // Upload/download address
  input  [31:0] idata;  // Download data word
  output [31:0] odata;  // Upload data word
  output [2:0]  status; // Exception status
  input clock;          // Clock input

// Define modes
  parameter RUN_MODE = 0;      // Normal operation
  parameter RESET_MODE = 1;    // Resetting processor;
  parameter DOWNLOAD_MODE = 2; // Transfering to memory
  parameter UPLOAD_MODE = 3;   // Reading from memory
  parameter STATUS_MODE = 4;   // Uploading register & other status information

// Constant values

  // Instruction functions
  parameter INOP =    4'h0;
  parameter IHALT =   4'h1;
  parameter IRRMOVL = 4'h2;
  parameter IIRMOVL = 4'h3;
  parameter IRMMOVL = 4'h4;
  parameter IMRMOVL = 4'h5;
  parameter IOPL =    4'h6;
  parameter IJXX =    4'h7;
  parameter ICALL =   4'h8;
  parameter IRET =    4'h9;
  parameter IPUSHL =  4'hA;
  parameter IPOPL =   4'hB;
  parameter IIADDL =  4'hC;
  parameter ILEAVE =  4'hD;
  parameter IPOP2 =   4'hE;

  parameter JUNCOND = 4'h0;

  // Register IDs
  parameter RESP =  4'h4;
  parameter REBP =  4'h5;
  parameter RNONE = 4'h8;

  // ALU operations
  parameter ALUADD = 4'h0;

  // Exception conditions
  parameter EBUB = 3'h0;
  parameter EAOK = 3'h1;
  parameter EHLT = 3'h2;
  parameter EADR = 3'h3;
  parameter EINS = 3'h4;
  parameter EPIP = 3'h5;

// Fetch stage
  wire [31:0] new_F_predPC, F_predPC, f_pc;
  wire        f_ok;
  wire [2:0]  f_exc;
  wire [47:0] f_instr;
  wire [3:0]  f_icode;
  wire [3:0]  new_D_icode;
  wire [3:0]  f_ifun;
  wire [3:0]  f_rA;
  wire [3:0]  f_rB;
  wire [31:0] f_valC;
  wire [31:0] f_valP;
  wire        need_regids;
  wire        need_valC;
  wire        instr_valid;
  wire F_stall, F_bubble;

// Decode stage
  wire [2:0]  D_exc;
  wire [31:0] D_pc;
  wire [3:0]  D_icode;
  wire [3:0]  D_ifun;
  wire [3:0]  D_rA;
  wire [3:0]  D_rB;
  wire [31:0] D_valC;
  wire [31:0] D_valP;

  wire [31:0] new_E_valA;
  wire [31:0] new_E_valB;

  wire [31:0] d_rvalA, d_valA;
  wire [31:0] d_rvalB, d_valB;
  wire [3:0]  d_dstE, new_E_dstE;
  wire [3:0]  d_dstM, new_E_dstM;
  wire [3:0]  d_srcA, new_E_srcA;
  wire [3:0]  d_srcB, new_E_srcB;
  wire D_stall, D_bubble;

// Execute stage
  wire [2:0]  E_exc;
  wire [31:0] E_pc;
  wire [3:0]  E_icode;
  wire [3:0]  E_ifun;
  wire [31:0] E_valC;
  wire [31:0] E_valA;
  wire [31:0] E_valB;
  wire [3:0]  E_dstE;
  wire [3:0]  E_dstM;
  wire [3:0]  E_srcA;
  wire [3:0]  E_srcB;

  wire [31:0] aluA;
  wire [31:0] aluB;
  wire        set_cc;
  wire [2:0]  cc;
  wire [2:0]  new_cc;
  wire [3:0]  alufun;
  wire        e_Bch;
  wire [31:0] e_valE;
  wire [31:0] new_M_valA;
  wire E_stall, E_bubble;

// Memory stage
  wire [2:0]  M_exc;
  wire [31:0] M_pc;
  wire [3:0]  M_icode;
  wire [3:0]  M_ifun;
  wire        M_Bch;
  wire [31:0] M_valE;
  wire [31:0] M_valA;
  wire [3:0]  M_dstE;
  wire [3:0]  M_dstM;

  wire [2:0]  m_exc;
  wire [31:0] mem_addr;
  wire [31:0] mem_data;
  wire        mem_read;
  wire        mem_write;
  wire [31:0] m_valM;
  wire M_stall, M_bubble;
  wire m_ok;

// Write-back stage
  wire [2:0]  W_exc;
  wire [31:0] W_pc;
  wire [3:0]  W_icode;
  wire [31:0] W_valE;
  wire [31:0] W_valM;
  wire [3:0]  W_dstE;
  wire [3:0]  W_dstM;
  wire [31:0] w_valE;
  wire [31:0] w_valM;
  wire [3:0]  w_dstE;
  wire [3:0]  w_dstM;
  wire W_stall;

// Debugging logic
  wire [31:0] eax, ecx, edx, ebx, esp, ebp, esi, edi;
  wire zf = cc[2];
  wire sf = cc[1];
  wire of = cc[0];

// Control signals
  wire resetting = (mode == RESET_MODE);
  wire uploading = (mode == UPLOAD_MODE);
  wire downloading = (mode == DOWNLOAD_MODE);
  wire running = (mode == RUN_MODE);
  wire getting_status = (mode == STATUS_MODE);

  assign status = W_exc == EBUB ? EAOK : W_exc;
  assign odata = 
   // When getting status, get either register or special status value
    getting_status ?  
     (udaddr == 0 ? eax :
      udaddr == 4 ? ecx :
      udaddr == 8 ? edx :
      udaddr == 12 ? ebx :
      udaddr == 16 ? esp :
      udaddr == 20 ? ebp :
      udaddr == 24 ? esi :
      udaddr == 28 ? edi :
      udaddr == 32 ? cc :
      udaddr == 36 ? W_pc : 0)
    : m_valM;

// Fetch stage
  preg #(32) F_predPC_reg(F_predPC, new_F_predPC, F_stall, F_bubble | resetting, 0, clock);
// Decode stage
  preg #(3)  D_exc_reg(D_exc, f_exc, D_stall, D_bubble | resetting, EBUB, clock); // Exception
  preg #(32) D_pc_reg(D_pc, f_pc, D_stall, D_bubble | resetting, 0, clock);
  preg #(4)  D_icode_reg(D_icode, new_D_icode, D_stall, D_bubble | resetting, INOP, clock);
  preg #(4)  D_ifun_reg(D_ifun, f_ifun, D_stall, D_bubble | resetting, 0, clock);
  preg #(4)  D_rA_reg(D_rA, f_rA, D_stall, D_bubble | resetting, RNONE, clock);
  preg #(4)  D_rB_reg(D_rB, f_rB, D_stall, D_bubble | resetting, RNONE, clock);
  preg #(32)  D_valC_reg(D_valC, f_valC, D_stall, D_bubble | resetting, 0, clock);
  preg #(32)  D_valP_reg(D_valP, f_valP, D_stall, D_bubble | resetting, 0, clock);
// Execute stage
  preg #(3)   E_exc_reg(E_exc, D_exc, E_stall, E_bubble | resetting, EBUB, clock); // Exception
  preg #(32)  E_pc_reg(E_pc, D_pc, E_stall, E_bubble | resetting, 0, clock);
  preg #(4)   E_icode_reg(E_icode, D_icode, E_stall, E_bubble | resetting, INOP, clock);
  preg #(4)   E_ifun_reg(E_ifun, D_ifun, E_stall, E_bubble | resetting, 0, clock);
  preg #(32)  E_valC_reg(E_valC, D_valC, E_stall, E_bubble | resetting, 0, clock);
  preg #(32)  E_valA_reg(E_valA, new_E_valA, E_stall, E_bubble | resetting, 0, clock);
  preg #(32)  E_valB_reg(E_valB, new_E_valB, E_stall, E_bubble | resetting, 0, clock);
  preg #(4)  E_dstE_reg(E_dstE, d_dstE, E_stall, E_bubble | resetting, RNONE, clock);
  preg #(4)  E_dstM_reg(E_dstM, d_dstM, E_stall, E_bubble | resetting, RNONE, clock);
  preg #(4)  E_srcA_reg(E_srcA, d_srcA, E_stall, E_bubble | resetting, RNONE, clock);
  preg #(4)  E_srcB_reg(E_srcB, d_srcB, E_stall, E_bubble | resetting, RNONE, clock);
// Memory stage
  preg #(3)   M_exc_reg(M_exc, E_exc, M_stall, M_bubble | resetting, EBUB, clock); // Exception
  preg #(32)  M_pc_reg(M_pc, E_pc, M_stall, M_bubble | resetting, 0, clock);
  preg #(4)   M_icode_reg(M_icode, E_icode, M_stall, M_bubble | resetting, INOP, clock);
  preg #(4)   M_ifun_reg(M_ifun, E_ifun, M_stall, M_bubble | resetting, 0, clock);
  preg #(1)   M_Bch_reg(M_Bch, e_Bch, M_stall, M_bubble | resetting, 0, clock);
  preg #(32)  M_valE_reg(M_valE, e_valE, M_stall, M_bubble | resetting, 0, clock);
  preg #(32)  M_valA_reg(M_valA, new_M_valA, M_stall, M_bubble | resetting, 0, clock);
  preg #(4)   M_dstE_reg(M_dstE, E_dstE, M_stall, M_bubble | resetting, RNONE, clock);
  preg #(4)   M_dstM_reg(M_dstM, E_dstM, M_stall, M_bubble | resetting, RNONE, clock);
// Write-back stage
  preg #(3)   W_exc_reg(W_exc, m_exc, W_stall, resetting, EBUB, clock); // Exception
  preg #(32)  W_pc_reg(W_pc, M_pc, W_stall, resetting, 0, clock);
  preg #(4)   W_icode_reg(W_icode, M_icode, W_stall, resetting, INOP, clock);
  preg #(32)  W_valE_reg(W_valE, M_valE, W_stall, resetting, 0, clock);
  preg #(32)  W_valM_reg(W_valM, m_valM, W_stall, resetting, 0, clock);
  preg #(4)   W_dstE_reg(W_dstE, M_dstE, W_stall, resetting, RNONE, clock);
  preg #(4)   W_dstM_reg(W_dstM, M_dstM, W_stall, resetting, RNONE, clock);

// Fetch stage logic
  split split(f_instr[7:0], f_icode, f_ifun);
  align align(f_instr[47:8], need_regids, f_rA, f_rB, f_valC);
  pc_increment pci(f_pc, need_regids, need_valC, f_valP);
  assign f_exc = !f_ok ? EADR : !instr_valid ? EINS : f_icode == IHALT ? EHLT : EAOK;

  // In normal design, icode passed directly from split block to FD pipeline register // OMIT_1w
  assign new_D_icode = f_icode;  // OMIT_1w
// Decode stage
  regfile regf(w_dstE, w_valE, w_dstM, w_valM, 
               d_srcA, d_rvalA, d_srcB, d_rvalB, resetting, clock,
               eax, ecx, edx, ebx, esp, ebp, esi, edi);
  assign d_srcA = new_E_srcA;
  assign d_srcB = new_E_srcB;
  assign d_dstE = new_E_dstE;
  assign d_dstM = new_E_dstM;

// Execute stage
  alu alu(aluA, aluB, alufun, e_valE, new_cc);
  cc  ccreg(cc, new_cc, 
	    // Only update CC when everything is running normally
            running && set_cc & !W_stall & (M_exc == EAOK || M_exc == EBUB), 
            resetting, clock);
  bcond bc(E_icode, E_ifun, cc, e_Bch);

  // In normal design, E_valA passed directly to the EM pipeline register // OMIT_lf
  assign new_M_valA = E_valA; // OMIT_lf

// Memory stage
  bmemory m(
    // Only update memory when everything is running normally or when downloading
    (running & mem_write & !W_stall) | downloading,  // When to write to memory
    downloading ? udaddr : mem_addr,             // Write address
    downloading ? idata : M_valA,                // Write data
    (running & mem_read) | uploading,                // When to read memory
    uploading ? udaddr : mem_addr,               // Read address
    m_valM,                                   // Read data
    m_ok,
    f_pc, f_instr, f_ok, clock);
  assign m_exc = (M_exc == EAOK | M_exc == EBUB) & !m_ok ? EADR : M_exc;
// Write-back stage logic
  assign W_stall = (W_exc != EAOK) & (W_exc != EBUB);

// Normal connections to register file  // OMIT_1w
  assign w_valE = W_valE;        // OMIT_1w
  assign w_valM = W_valM;        // OMIT_1w
  assign w_dstE = W_dstE;        // OMIT_1w
  assign w_dstM = W_dstM;        // OMIT_1w

// Glue logic
// --------------------------------------------------------------------
// The following code is generated from the HCL description of the
// pipeline control using a program hcl2v
// --------------------------------------------------------------------
// INSERT_LOGIC_HERE
// --------------------------------------------------------------------
// End of code generated by hcl2v
// --------------------------------------------------------------------
endmodule
// --------------------------------------------------------------------
// End of file "pipe-proc.v"
// --------------------------------------------------------------------





