
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>

/* $begin tadd_ok_proto */
/* Determine whether arguments can be added without overflow */
int tadd_ok(int x, int y);
/* $end tadd_ok_proto */

/* $begin tadd_ok */
/* Determine whether arguments can be added without overflow */
int tadd_ok(int x, int y) {
    int sum = x+y;
    int neg_over = x <  0 && y <  0 && sum >= 0;
    int pos_over = x >= 0 && y >= 0 && sum <  0;
    return !neg_over && !pos_over;
}
/* $end tadd_ok */

#if 0
/* $begin bad_tadd_ok */
/* Determine whether arguments can be added without overflow */
/* WARNING: This code is buggy. */
int tadd_ok(int x, int y) {
    int sum = x+y;
    return (sum-x == y) && (sum-y == x);
}
/* $end bad_tadd_ok */
#endif


#if 0
/* $begin bad_tsub_ok */
/* Determine whether arguments can be subtracted without overflow */
/* WARNING: This code is buggy. */
int tsub_ok(int x, int y) {
    return tadd_ok(x, -y);
}
/* $end bad_tsub_ok */
#endif

/* $begin tsub_ok_proto */
/* Determine whether arguments can be subtracted without overflow */
int tsub_ok(int x, int y);
/* $end tsub_ok_proto */

/* $begin tsub_ok */
/* Determine whether arguments can be subtracted without overflow */
int tsub_ok(int x, int y) {
    int diff = x-y;
    int neg_over = x <  0 && y >= 0 && diff >= 0;
    int pos_over = x >= 0 && y <  0 && diff <  0;
    return !neg_over && !pos_over;
}
/* $end tsub_ok */

/* $begin tadd_ovf_proto */
/* Determine whether adding arguments will cause overflow */
int tadd_ovf(int x, int y);
/* $end tadd_ovf_proto */

/* $begin tadd_ovf */
/* Determine whether adding arguments will cause overflow */
int tadd_ovf(int x, int y) {
    int sum = x+y;
    int neg_over = x <  0 && y <  0 && sum >= 0;
    int pos_over = x >= 0 && y >= 0 && sum <  0;
    return neg_over || pos_over;
}
/* $end tadd_ovf */

#if 0
/* $begin bad_tadd_ovf */
/* Determine whether adding arguments will cause overflow */
/* WARNING: This code is buggy. */
int tadd_ovf(int x, int y) {
    int sum = x+y;
    return (sum-x != y) || (sum-y != x);
}
/* $end bad_tadd_ovf */
#endif

#if 0
/* $begin bad_tsub_ovf */
/* Determine whether subtracting arguments will cause overflow */
/* WARNING: This code is buggy. */
int tsub_ovf(int x, int y) {
    return tadd_ovf(x, -y);
}
/* $end bad_tsub_ovf */
#endif

/* $begin tsub_ovf_proto */
/* Determine whether subtracting arguments will cause overflow */
int tsub_ovf(int x, int y);
/* $end tsub_ovf_proto */

/* $begin tsub_ovf */
/* Determine whether subtracting arguments will cause overflow */
int tsub_ovf(int x, int y) {
    int diff = x-y;
    int neg_over = x <  0 && y >= 0 && diff >= 0;
    int pos_over = x >= 0 && y <  0 && diff <  0;
    return neg_over || pos_over;
}
/* $end tsub_ovf */

/* Test function */
int test_tadd_ok(int x, int y)
{
    long long lx = (long long) x;
    long long ly = (long long) y;
    return (lx + ly) == (x + y);
}


/* Test function */
int test_tadd_ovf(int x, int y)
{
    long long lx = (long long) x;
    long long ly = (long long) y;
    return (lx + ly) != (x + y);
}

int main(int argc, char *argv[]) {
    int x = argc > 1 ?  strtoll(argv[1], NULL, 0) : INT_MIN;
    int y = argc > 1 ?  strtoll(argv[2], NULL, 0) : INT_MAX;
    int ok = tadd_ok(x, y);
    int ovf = tadd_ovf(x, y);
    int tok = test_tadd_ok(x, y);
    int tovf = test_tadd_ovf(x, y);
    printf("x = %d, y = %d, x+y = %d\n", x, y, x+y);
    if (ok != tok)
	printf("\tok = %d, tok = %d\n", ok, tok);
    if (ovf != tovf)
	printf("\tovf = %d, tovf = %d\n", ovf, tovf);
    return 0;
}
