#!/usr/bin/perl   
#!/usr/local/bin/perl 
use Getopt::Std;
use lib ".";
use config;

#########################################################################
# grade-archlaba.pl - Architecture Lab Part A autograder
#
# Copyright (c) 2002, R. Bryant and D. O'Hallaron, All rights reserved.
# May not be used, modified, or copied without permission.
#########################################################################

# autoflush output on every print statement
$| = 1; 

# Any tmp files created by this script are readable only by the user
umask(0077); 

#
# usage - print help message and terminate
#
sub usage {
    printf STDERR "$_[0]\n";
    printf STDERR "Usage: $0 -f <file> [-he] [-s <codedir>]\n";
    printf STDERR "Options:\n";
    printf STDERR "  -h           Print this message.\n";
    printf STDERR "  -e           Don't include original handin file on the grade sheet\n";
    printf STDERR "  -f <file>    Prefix of handin files to be graded\n";
    printf STDERR "  -s <codedir> Directory where the Y86 tools are located\n";
    die "\n";
}

##############
# Main routine
##############

# 
# Parse the command line arguments
#
getopts('hef:s:');
if ($opt_h) {
    usage();
}
if (!$opt_f) {
    usage("Missing required argument (-f)");
}

# 
# These optional flags override defaults in config.pm
#
if ($opt_s) {         # simulator directory
    $CODEDIR = $opt_s;
}


# 
# Initialize some file and path names
#
$infile = $opt_f;
($infile_basename = $infile) =~ s#.*/##s;     # basename of input files
$tmpdir = "/tmp/$infile_basename.$$";         # scratch directory
$0 =~ s#.*/##s;                               # this prog's basename

# absolute pathname of src directory
$simdir_abs = `cd $CODEDIR; pwd`; 
chomp($simdir_abs);

# 
# This is a message we use in several places when the program dies
#
$diemsg = "The files are in $tmpdir.";

# 
# Make sure the yis and yas binaries exist
#
(-e "$simdir_abs/misc/yas" and -x "$simdir_abs/misc/yas")
    or  die "$0: ERROR: Can't access $simdir_abs/misc/yas.\n";
(-e "$simdir_abs/misc/yis" and -x "$simdir_abs/misc/yis")
    or  die "$0: ERROR: Can't access $simdir_abs/misc/yis.\n";

# 
# Make sure all of the input handin files exist and are readable
#
foreach $prog (@PROGS) {
    $filename = "$infile-$prog.ys";
    open(INFILE, "$filename")  
	or die "$0: ERROR: could not open handin file $filename\n";
    close(INFILE);
}

# 
# Set up the contents of the scratch directory
#
system("mkdir $tmpdir") == 0
  or die "ERROR: mkdir $tmpdir failed\n";
foreach $prog (@PROGS) {
    $prefix = "$infile-$prog";
    system("cp $prefix.ys $tmpdir/$prog.ys") == 0
	or die "ERROR: cp $prefix.ys to $tmpdir failed\n";
}

# Print header
print "\nCS:APP Architecture Lab Part A: Grading Sheet for $infile_basename\n\n";

#
# Compile and run each example program
# 
#
$partnum = 1;
foreach $prog (@PROGS) {
    $prefix = "$prog";
    print "***************\n";
    print "Part $partnum: $prefix.ys\n";
    print "***************\n";
    print "\n";
    
    if (!$opt_e) {
      system("(cd $tmpdir; cat $prefix.ys)") == 0 
	or die "$0: ERROR: Unable to emit $prefix.ys. $diemsg\n";
    }
    print "\nRunning yas $prefix.ys:\n";
    system("(cd $tmpdir; $simdir_abs/misc/yas $prefix.ys)") == 0 
	or die "$0: ERROR: Unable to compile $prefix.ys. $diemsg\n";
    print "OK\n";
    print "\nRunning yis $prefix.yo:\n";
    system("(cd $tmpdir; $simdir_abs/misc/yis $prefix.yo > $prefix.out)") == 0 
	or die "$0: ERROR: Unable to simulate $prefix.yo. $diemsg\n";
    system("(cd $tmpdir; cat $prefix.out)") == 0 
	or die "$0: ERROR: Unable to print $prefix.out. $diemsg\n";

    if ($partnum <= $#PROGS) {
	print "\f\n";
    }
    $partnum++;
}

#
# Everything went OK, so remove the scratch directory
#
system("rm -fr $tmpdir");

exit;


